/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.plot.simple.series;

import java.io.Serializable;
import java.util.Objects;

public class XYDataErrorItem implements Cloneable, Serializable {

	private static final long serialVersionUID = 4816792151317720492L;
	private double x;
	private double yLow;
	private double yHigh;


	/**
	 * Constructs a new error item.
	 *
	 * @param x the x value.
	 * @param yLow the y low error value.
	 * @param yHigh the y high error value.
	 */
	public XYDataErrorItem(double x, double yLow, double yHigh) {
		this.x = x;
		this.yLow = yLow;
		this.yHigh = yHigh;
	}

	/**
	 * Returns the yLow value for error.
	 *
	 * @return The yLow value for error.
	 */
	public double getYLowErrorValue() {
		return yLow;
	}

	/**
	 * Returns the yHigh value for error.
	 *
	 * @return The yHigh value for error.
	 */
	public double getYHighErrorValue() {
		return yHigh;
	}
 
	/**
	 * Sets the yLow value for error for this data item.
	 *
	 * @param yLow the yLow value for error.
	 */
	public void setYLowError(double yLow) {
		this.yLow = yLow;
	}

	/**
	 * Sets the yHigh value for error for this data item.
	 *
	 * @param yHigh the yHigh value for error.
	 */
	public void setYHighError(double yHigh) {
		this.yHigh = yHigh;
	}
	
	/**
	 * Returns the x value.
	 *
	 * @return The x value.
	 */
	public double getX() {
		return x;
	}

	/**
	 * Sets the x value for this data item.
	 *
	 * @param x the x value.
	 */
	public void setX(double x) {
		this.x = x;
	}

    /**
     * Returns a clone of this object.
     *
     * @return A clone.
     */
    @Override
    public Object clone() {
        return new XYDataErrorItem(x, yLow, yHigh);
    }

	@Override
	public int hashCode() {
		return Objects.hash(x, yHigh, yLow);
	}

    /**
     * Tests if this object is equal to another.
     *
     * @param obj  the object to test against for equality (<code>null</code>
     *             permitted).
     *
     * @return A boolean.
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof XYDataErrorItem)) {
            return false;
        }
        XYDataErrorItem that = (XYDataErrorItem) obj;
        if (Double.compare(this.x, that.x) != 0
        		|| Double.compare(this.yLow, that.yLow) != 0
        		|| Double.compare(this.yHigh, that.yHigh) != 0) {
        	return false;
        }
        return true;
    }


}
